﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.PluginInfrastructure;
using System.ServiceModel;
using HeuristicLab.Services.Hive.Common.ServiceContracts;
using System.Windows.Forms;

namespace HeuristicLab.Services.Hive.Standalone {
  [Application("Hive Server", "Server application for the distributed hive engine.", false)]
  public class HeuristicLabServicesHiveApplication : ApplicationBase {
    private IDictionary<string, ServiceHost> serviceHosts = new Dictionary<string, ServiceHost>();
    private Hive hive { get { return ServiceLocator.Instance.Hive; } }

    public override void Run() {
      try {
        CreateAndOpenServiceHost(typeof(IHiveService), "HiveService");
        Form mainForm = new MainForm(GetBaseAddresses());
        Application.Run();
      }
      finally {
        foreach (ServiceHost host in serviceHosts.Values) {
          if (host.State == CommunicationState.Opened)
            host.Close();
        }
        hive.Shutdown();
      }
    }

    private ServiceHost CreateAndOpenServiceHost(Type serviceInterfaceType, string serviceName) {
      ServiceHost host = new ServiceHost(ApplicationManager.Manager.GetTypes(serviceInterfaceType).First());
      host.Open();
      serviceHosts.Add(serviceName, host);
      return host;
    }

    private IEnumerable<Uri> GetBaseAddresses() {
      List<Uri> list = new List<Uri>();
      foreach (KeyValuePair<string, ServiceHost> host in serviceHosts) {
        foreach (var addr in host.Value.BaseAddresses) {
          list.Add(addr);
        }
      }
      return list;
    }
  }
}
